/* 
SVN Revisionsinformationen
$Revision: 713 $
$Date: 2017-03-14 08:13:39 +0100 (Di, 14 Mrz 2017) $  
*/ 

/*
drop table D_ADRESSE cascade constraints;
drop table D_ABTEILUNG cascade constraints;
drop table D_BETRIEBSSTAETTE cascade constraints;
drop table D_PERSONALFUNKTION cascade constraints;
drop table D_EXPORTSPERRE cascade constraints;
drop table D_BETRIEBSART cascade constraints;
drop table D_PERSONAL cascade constraints;
drop table D_PROTOKOLL cascade constraints;
drop table D_REGISTRIERUNG cascade constraints;
drop table D_FACHBEREICH cascade constraints;
drop table D_USECASE cascade constraints;

drop TABLE WKD_BEA;

*/

DECLARE 
  vCnt INT;
  vIDX_TBS VARCHAR2(50) := NULL; 
  vUI_TBS VARCHAR2(60) := NULL; 
        
  FUNCTION FieldExists(pTable_name IN VARCHAR2, pColumn_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tab_columns
    WHERE table_name = pTable_name
      AND column_name = pColumn_name;
      
    RETURN vCnt > 0; 
  
  END;

  FUNCTION TableExists(pTable_name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_tables
    WHERE table_name = pTable_name;
      
    RETURN vCnt > 0; 
  
  END;
  
  FUNCTION IdxExists(pIndex_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_indexes
    WHERE index_name = pIndex_Name;
      
    RETURN vCnt > 0; 
  
  END;  

  PROCEDURE COMMENT_COL(pTab_Col IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON COLUMN ' || pTab_Col || ' IS ''' || pComment || '''';  
  END;
  
  PROCEDURE COMMENT_TAB(pTab IN VARCHAR2, pComment IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE 'COMMENT ON TABLE ' || pTab || ' IS ''' || pComment || '''';  
  END;

  FUNCTION ConstraintExists(pConstraint_Name IN VARCHAR2) RETURN BOOLEAN AS
  BEGIN

    SELECT COUNT(*) INTO vCnt
    FROM user_constraints
    WHERE constraint_name = pConstraint_name;
      
    RETURN vCnt > 0; 
  
  END;

  PROCEDURE CrtIdx(pSQL IN VARCHAR2) AS
  BEGIN
    EXECUTE IMMEDIATE pSQL || vIDX_TBS;
  END;
                
BEGIN

  EXECUTE IMMEDIATE 'ALTER SESSION SET NLS_LENGTH_SEMANTICS = CHAR';
 
  -- Index-Tablespace merken
  SELECT tablespace_name INTO vIDX_TBS
  FROM user_indexes
  WHERE index_name = 'PK_D0101_ID';
  
  IF vIDX_TBS IS NOT NULL THEN
    vIDX_TBS := ' TABLESPACE ' || vIDX_TBS;
    vUI_TBS := ' USING INDEX ' || vIDX_TBS;
  END IF;

  IF NOT TableExists('D_ADRESSE') THEN
    EXECUTE IMMEDIATE '
    
      CREATE  TABLE D_ADRESSE
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           FREITEXT VARCHAR2(50) NULL ,
           WECHSELNDER_STANDORT VARCHAR2(255) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           TYP VARCHAR2(10) DEFAULT ''S'' NULL ,
           ADRESSZUSATZ VARCHAR2(255) NULL ,
           GEMEINDE VARCHAR2(50) NULL ,
           GKZ VARCHAR2(14) NULL ,
           REGIONALSCHLUESSEL VARCHAR2(50) DEFAULT ''0000000'' NULL ,
           STRASSE VARCHAR2(50) NULL ,
           HAUSNUMMER VARCHAR2(20) NULL ,
           HAUSNUMMERNZUSATZ VARCHAR2(20) NULL ,
           POSTFACH VARCHAR2(50) NULL ,
           STAAT VARCHAR2(3) NULL ,
           PLZ VARCHAR2(10) NULL ,
           ORT VARCHAR2(50) NULL ,
           ORTSTEIL VARCHAR2(50) NULL ,
           TEILORT VARCHAR2(50) NULL ,
           FLURNUMMER VARCHAR2(20) NULL ,
           FLURSTUECK VARCHAR2(20) NULL ,
           GEMARKUNG VARCHAR2(50) NULL ,
           X_WERT FLOAT NULL ,
           Y_WERT FLOAT NULL ,
           RECHTSWERT FLOAT NULL ,
           HOCHWERT FLOAT NULL ,
           KOORDINATENHERKUNFT VARCHAR2(10) NULL ,
           S09_SID NUMBER(10) DEFAULT (-1) NOT NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           GLO_A24_SID NUMBER(10) NULL ,
           POSTBEZEICHNUNG VARCHAR2(255) NULL      ,
           CONSTRAINT PK_D_ADR$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';


      Comment_Tab('D_ADRESSE', 'Adressen (Krzel ADR)');
      Comment_Col('D_ADRESSE.FREITEXT', 'Freitext');
      Comment_Col('D_ADRESSE.WECHSELNDER_STANDORT', 'Wechselnder Standort');
      Comment_Col('D_ADRESSE.GUELTIG_VON', 'gltig von');
      Comment_Col('D_ADRESSE.GUELTIG_BIS', 'gltig bis');
      Comment_Col('D_ADRESSE.TYP', 'Adressart: S: Strasse (STREET), P: Postfach (POST_OFFICE_BOX, F: freie Bezeichnung (free description) . V: wechselnder Standort ( VARIABLE_LOCATION)  (ART)');
      Comment_Col('D_ADRESSE.ADRESSZUSATZ', 'Zusatz');
      Comment_Col('D_ADRESSE.GEMEINDE', 'Beinhaltet fr BTR aus der GKZ die Gemeinde, fr BEA Bezeichnung der Gemeinde');
      Comment_Col('D_ADRESSE.REGIONALSCHLUESSEL', 'Regionalschlssel');
      Comment_Col('D_ADRESSE.STRASSE', 'Strae');
      Comment_Col('D_ADRESSE.HAUSNUMMER', 'Hausnummer');
      Comment_Col('D_ADRESSE.HAUSNUMMERNZUSATZ', 'Hausnr.-Zusatz');
      Comment_Col('D_ADRESSE.POSTFACH', 'Postfach');
      Comment_Col('D_ADRESSE.STAAT', 'GLO_A24.POST (STAAT)');
      Comment_Col('D_ADRESSE.PLZ', 'PLZ');
      Comment_Col('D_ADRESSE.ORT', 'Ort');
      Comment_Col('D_ADRESSE.ORTSTEIL', 'Ortsteil');
      Comment_Col('D_ADRESSE.TEILORT', 'Teilort');
      Comment_Col('D_ADRESSE.FLURNUMMER', 'Flurnummer');
      Comment_Col('D_ADRESSE.FLURSTUECK', 'Flurstck');
      Comment_Col('D_ADRESSE.GEMARKUNG', 'Gemarkung');
      Comment_Col('D_ADRESSE.X_WERT', 'X-Wert');
      Comment_Col('D_ADRESSE.Y_WERT', 'Y-WERT');
      Comment_Col('D_ADRESSE.RECHTSWERT', 'Rechstwert');
      Comment_Col('D_ADRESSE.HOCHWERT', 'Hochwert');
      Comment_Col('D_ADRESSE.KOORDINATENHERKUNFT', 'Herkunft der Koordinaten: TSN, HK, MANUELL (GIS-Quelle)');
      Comment_Col('D_ADRESSE.S09_SID', 'Referenz auf Katalog Koordinatensystem');
      Comment_Col('D_ADRESSE.GLO_A24_SID', 'Referenz auf Katalog Land');
      Comment_Col('D_ADRESSE.POSTBEZEICHNUNG', 'abweichende Postbezeichnung');
  END IF;
  
  IF NOT TableExists('D_BETRIEBSART') THEN

    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_BETRIEBSART
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           BTS_ID VARCHAR2(32) NOT NULL ,
           TYP VARCHAR2(50) NULL ,
           CODE VARCHAR2(50) NULL ,
           BEZEICHNUNG VARCHAR2(255) NULL ,
           FB VARCHAR2(50) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           D010101_ID VARCHAR2(32) NULL ,
           GELTUNGSBEREICH VARCHAR2(32) DEFAULT ''NOT_VALID'' NULL ,
           GEWERBSMAESSIG_JN NUMBER(10) DEFAULT 0 NOT NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           ADR_STO_ID VARCHAR2(32) NULL      ,
           CONSTRAINT PK_D_BAR$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE  ' || vUI_TBS || '
      )';


      CrtIdx('CREATE INDEX IDX_D_BAR$BTS_ID ON D_BETRIEBSART (BTS_ID)');
      CrtIdx('CREATE INDEX IDX_D_BAR$ADR_STO_ID ON D_BETRIEBSART (ADR_STO_ID)');
      Comment_Tab('D_BETRIEBSART', 'Betriebsarten (Krzel BAR)');
      Comment_Col('D_BETRIEBSART.TYP', 'Art');
      Comment_Col('D_BETRIEBSART.BEZEICHNUNG', 'Bezeichnung');
      Comment_Col('D_BETRIEBSART.FB', 'Fachbereiche');
      Comment_Col('D_BETRIEBSART.GUELTIG_VON', 'Anmeldung');
      Comment_Col('D_BETRIEBSART.GUELTIG_BIS', 'Abmeldung');
      Comment_Col('D_BETRIEBSART.D010101_ID', 'Referenz zu VIS_D010101.ID');
      Comment_Col('D_BETRIEBSART.GEWERBSMAESSIG_JN', 'gewerbsmig J/N');
      Comment_Col('D_BETRIEBSART.ADR_STO_ID', 'Abweichender Standort der Betriebsart');
  END IF;
  
  IF NOT TableExists('D_BETRIEBSSTAETTE') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_BETRIEBSSTAETTE
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           ADR_PST_ID VARCHAR2(32) NULL ,
           ADR_STO_ID VARCHAR2(32) NULL ,
           ADR_BTR_ID VARCHAR2(32) NULL ,
           ADR_STD_ID VARCHAR2(32) NULL ,
           BEZEICHNUNG VARCHAR2(255) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           D0101_ID VARCHAR2(32) NULL ,
           ZB VARCHAR2(5) NULL ,
           KENNZEICHEN VARCHAR2(128) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           TELEFON VARCHAR2(128) NULL ,
           FAX VARCHAR2(128) NULL ,
           INTERNET VARCHAR2(128) NULL ,
           MOBIL VARCHAR2(128) NULL ,
           EMAIL VARCHAR2(128) NULL ,
           WEITERE VARCHAR2(512) NULL      ,
           CONSTRAINT PK_D_BTS$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';
    CrtIdx('CREATE INDEX IDX_D_BTS$ADR_STO_ID ON D_BETRIEBSSTAETTE (ADR_STO_ID)');
    CrtIdx('CREATE INDEX IDX_D_BTS$ADR_PST_ID ON D_BETRIEBSSTAETTE (ADR_PST_ID)');
    CrtIdx('CREATE INDEX IDX_D_BTS$ADR_BTR_ID ON D_BETRIEBSSTAETTE (ADR_BTR_ID)');
    CrtIdx('CREATE INDEX IDX_D_BTS$ADR_STD_ID ON D_BETRIEBSSTAETTE (ADR_STD_ID)');
    Comment_Tab('D_BETRIEBSSTAETTE', 'Betriebssttten (Krzel BTS)');
    Comment_Col('D_BETRIEBSSTAETTE.ADR_PST_ID', 'Postadresse (Post address)');
    Comment_Col('D_BETRIEBSSTAETTE.ADR_STO_ID', 'Standortadresse (Location address)');
    Comment_Col('D_BETRIEBSSTAETTE.ADR_BTR_ID', 'Betreiber (Operator address)');
    Comment_Col('D_BETRIEBSSTAETTE.ADR_STD_ID', 'Standardadresse fr Anschreiben (Default Address)');
    Comment_Col('D_BETRIEBSSTAETTE.BEZEICHNUNG', 'Zusatzbezeichnung / Standortbezeichnung');
    Comment_Col('D_BETRIEBSSTAETTE.GUELTIG_VON', 'Anmeldung (gltig von)');
    Comment_Col('D_BETRIEBSSTAETTE.GUELTIG_BIS', 'Abmeldung (gtlig bis)');
    Comment_Col('D_BETRIEBSSTAETTE.D0101_ID', 'Referenz zu VIS_D0101.ID');
    Comment_Col('D_BETRIEBSSTAETTE.WEITERE', 'sonstige Kommunikationsmittel');
  END IF;
  
  IF NOT TableExists('D_PERSONALFUNKTION') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_PERSONALFUNKTION
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           TYP VARCHAR2(20) DEFAULT ''BTR'' NULL ,
           PRS_ID VARCHAR2(32) NOT NULL ,
           BTS_ID VARCHAR2(32) NOT NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL      ,
           CONSTRAINT PK_D_PFK$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';
    CrtIdx('CREATE INDEX IDX_D_PFK$BTS_ID ON D_PERSONALFUNKTION (BTS_ID)');
    CrtIdx('CREATE INDEX IDX_D_PFK$PRS_ID ON D_PERSONALFUNKTION (PRS_ID)');
    Comment_Tab('D_PERSONALFUNKTION', 'Funktion (Krzel PFK) Betreiber, Hauptverantwortliche (immer natrliche Person NP)');
    Comment_Col('D_PERSONALFUNKTION.TYP', 'Betreiber (Operator) BTR, Hauptverantwortlicher (Person in charge) HV, weitere Personen (natural person) NP');
  END IF;
  
  IF NOT TableExists('D_PERSONAL') THEN
    EXECUTE IMMEDIATE '         
      CREATE  TABLE D_PERSONAL
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           ADR_STZ_ID VARCHAR2(32) NULL ,
           ADR_PST_ID VARCHAR2(32) NULL ,
           OWNER_PRS_ID VARCHAR2(32) NULL ,
           OWNER_TYPE VARCHAR2(20) NULL ,
           KURZBEZEICHNUNG1 VARCHAR2(50) NULL ,
           KURZBEZEICHNUNG2 VARCHAR2(50) NULL ,
           TYP VARCHAR2(2) DEFAULT ''JP'' NULL ,
           ANREDE VARCHAR2(50) NULL ,
           TITEL VARCHAR2(50) NULL ,
           VORNAME VARCHAR2(128) NULL ,
           NACHNAME VARCHAR2(128) NULL ,
           GEBURTSDATUM DATE NULL ,
           GEBURTSORT VARCHAR2(50) NULL ,
           BEZEICHNUNG VARCHAR2(128) NULL ,
           BEZEICHNUNG2 VARCHAR2(128) NULL ,
           RECHTSFORM VARCHAR2(128) NULL ,
           PERS_ID VARCHAR2(20) DEFAULT ''0'' NULL ,
           REF_ID VARCHAR2(32) NULL ,
           VORSATZ_NACHNAME VARCHAR2(20) NULL ,
           NACHSATZ_NACHNAME VARCHAR2(20) NULL ,
           A0101_03_SID NUMBER(10) NULL ,
           A0101_70_SID NUMBER(10) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           TELEFON VARCHAR2(128) NULL ,
           FAX VARCHAR2(128) NULL ,
           INTERNET VARCHAR2(128) NULL ,
           MOBIL VARCHAR2(128) NULL ,
           EMAIL VARCHAR2(128) NULL ,
           WEITERE VARCHAR2(512) NULL ,
           HAUPTANSPRECHPARTNER_JN NUMBER(1) NULL    ,
           CONSTRAINT PK_D_PRS$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_D_PRS$OWN_PRS_ID ON D_PERSONAL (OWNER_PRS_ID)');
    CrtIdx('CREATE INDEX IDX_D_PRS$ADR_STZ_ID ON D_PERSONAL (ADR_STZ_ID)');
    CrtIdx('CREATE INDEX IDX_D_PRS$ADR_PST_ID ON D_PERSONAL (ADR_PST_ID)');
    Comment_Tab('D_PERSONAL', 'juristische und natrliche Personen (Krzel PRS)');
    Comment_Col('D_PERSONAL.ADR_STZ_ID', 'Sitzadresse (Registered office)');
    Comment_Col('D_PERSONAL.ADR_PST_ID', 'Postadresse (Post address)');
    Comment_Col('D_PERSONAL.OWNER_TYPE', 'Funktionstyp der Beziehung:  Betreiber OPERATOR / Personal STAFF');
    Comment_Col('D_PERSONAL.KURZBEZEICHNUNG1', 'Kurbez1');
    Comment_Col('D_PERSONAL.KURZBEZEICHNUNG2', 'Kurzbez2');
    Comment_Col('D_PERSONAL.TYP', 'NP - natrliche Person  (Natural Person), JP - juristische Person (Legal Person)');
    Comment_Col('D_PERSONAL.ANREDE', 'Anrede');
    Comment_Col('D_PERSONAL.TITEL', 'Titel');
    Comment_Col('D_PERSONAL.VORNAME', 'Vorname');
    Comment_Col('D_PERSONAL.NACHNAME', 'Nachname');
    Comment_Col('D_PERSONAL.GEBURTSDATUM', 'Geburtsdatum');
    Comment_Col('D_PERSONAL.GEBURTSORT', 'Geburtsort');
    Comment_Col('D_PERSONAL.BEZEICHNUNG', 'Bezeichnung');
    Comment_Col('D_PERSONAL.BEZEICHNUNG2', 'Bezeichnung2');
    Comment_Col('D_PERSONAL.RECHTSFORM', 'Rechtsform');
    Comment_Col('D_PERSONAL.PERS_ID', 'Referenzfeld fr NRW');
    Comment_Col('D_PERSONAL.REF_ID', 'VIS_D0101.ID fr TYP = JP und VIS_D0102 fr TYP = NP');
    Comment_Col('D_PERSONAL.VORSATZ_NACHNAME', 'Vorsatz Nachname');
    Comment_Col('D_PERSONAL.NACHSATZ_NACHNAME', 'Nachsatz Nachname');
    Comment_Col('D_PERSONAL.A0101_03_SID', 'Referenzkatalog Rechtsform');
    Comment_Col('D_PERSONAL.A0101_70_SID', 'Referenzkatalog Personalfunktion');
    Comment_Col('D_PERSONAL.TELEFON', 'Telefon');
    Comment_Col('D_PERSONAL.INTERNET', 'Internet');
    Comment_Col('D_PERSONAL.WEITERE', 'sonstige Kommunikationsmittel');
    Comment_Col('D_PERSONAL.HAUPTANSPRECHPARTNER_JN', 'Hauptansprechpartner');

  END IF;
  
  IF NOT TableExists('D_PROTOKOLL') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_PROTOKOLL
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           BTS_ID VARCHAR2(32) NOT NULL ,
           D0101_ID VARCHAR2(32) NOT NULL ,
           STATUS NUMBER(10) DEFAULT 0 NOT NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           USER_SID NUMBER(10) NOT NULL ,
           LETZTE_AENDERUNG DATE NOT NULL ,
           MANDANT NUMBER(10) DEFAULT (-1) NOT NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           PARAMETER CLOB NULL ,
           DATENHERKUNFT VARCHAR2(30) NOT NULL,
           UPD_USER NUMBER(10) DEFAULT -1 NOT NULL,
           UPD_DATE DATE DEFAULT SYSDATE NULL,     
           CONSTRAINT PK_D_PRT$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';


    CrtIdx('CREATE INDEX IDX_D_PRT$BTS_ID ON D_PROTOKOLL (BTS_ID)');
    CrtIdx('CREATE INDEX IDX_D_PRT$D0101_ID ON D_PROTOKOLL (D0101_ID,LETZTE_AENDERUNG,INS_DATE)');
    Comment_Tab('D_PROTOKOLL', 'Protokoll (Krzel PRT)');

    Comment_Col('D_PROTOKOLL.STATUS', 'Betriebsregister Ausgangskorb
    0 - nicht verarbeitet, vor dem Vergleich, 
    2 - gesendet, zum Vergleich, 
    3 - nur zum Vergleich, wird nicht und wurde nicht gesendet, 
    4 - nur zum Vergleich mit dem Eingangskorb, 
    5 - neuer Betrieb ohne Dublettenabgleich senden, 
    6 - ohne Plausiprfung

    Betriebsregister Eingangskorb
    0 - externe nderungen/Neuanlagen,  
    1 - Besttigung eigenen nderungen/Neuanlagen, 
    2 - Besttigung ANL_MIN als Dublette, 
    10 - Ablehnung, 
    20 - Betrieb als Dublette

    BEA:
    0 - neuer Betrieb/Betreiber
    1 - nderung Betreibe/Betreiber');


    Comment_Col('D_PROTOKOLL.DELETED', '0 = aktiv, -1 = inaktiv, -2 = gelscht');
    Comment_Col('D_PROTOKOLL.PARAMETER', 'spezielle Verfahrensparameter');
    Comment_Col('D_PROTOKOLL.DATENHERKUNFT', 'Datenherkunft: BEA_AK Ausgangskorb BEA, BEA_EK Eingangskorb BEA, BTR_AK Ausgangskorb Betriebsregister, BTR_EK Eingangskorb Betriebsregister');

  END IF;
  
  IF NOT TableExists('D_REGISTRIERUNG') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_REGISTRIERUNG
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           PRS_ID VARCHAR2(32) NULL ,
           BTS_ID VARCHAR2(32) NULL ,
           TYP VARCHAR2(50) NULL ,
           CODE VARCHAR2(50) NULL ,
           NUMMER VARCHAR2(50) NULL ,
           ZUISATZ VARCHAR2(255) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           REF_ID VARCHAR2(32) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL      ,
           CONSTRAINT PK_D_REG$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';


    CrtIdx('CREATE INDEX IDX_D_REG$BTS_ID ON D_REGISTRIERUNG (BTS_ID)');
    CrtIdx('CREATE INDEX IDX_D_REG$PRS_ID ON D_REGISTRIERUNG (PRS_ID)');
    Comment_Tab('D_REGISTRIERUNG', 'Registrierungen Betreiber oder Betriebssttte (Krzel REG)');
    Comment_Col('D_REGISTRIERUNG.ZUISATZ', 'Zusatz');
    Comment_Col('D_REGISTRIERUNG.GUELTIG_VON', 'gltig von');
    Comment_Col('D_REGISTRIERUNG.GUELTIG_BIS', 'gtig bis');
    Comment_Col('D_REGISTRIERUNG.REF_ID', 'Referenz zu VIS_D010102.ID, bei ART=BALVI zu VIS_D0101.ID');

  END IF;
  
  IF NOT TableExists('D_USECASE') THEN
    EXECUTE IMMEDIATE '   
      CREATE  TABLE D_USECASE
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           BTS_ID VARCHAR2(32) NULL ,
           BTS_ID2 VARCHAR2(32) NULL ,
           D0101_ID VARCHAR2(32) NOT NULL ,
           USECASE VARCHAR2(20) NOT NULL ,
           MANDANT NUMBER(10) NOT NULL ,
           USER_NAME VARCHAR2(128) NULL ,
           USER_SID NUMBER(10) NULL ,
           LETZTE_AENDERUNG DATE NOT NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           TYP NUMBER(10) DEFAULT 4 NOT NULL ,
           AENDERUNGSGRUND VARCHAR2(20) NOT NULL ,
           BEMERKUNG VARCHAR2(4000) NULL ,
           ALTER_WERT VARCHAR2(2000) NULL ,
           NEUER_WERT VARCHAR2(2000) NULL ,
           FEHLERMELDUNG CLOB NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           INS_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           DEl_USER NUMBER(10) DEFAULT (-1) NOT NULL ,
           DEL_DATE DATE NULL ,
           FB VARCHAR2(32) DEFAULT ''%'' NOT NULL ,
           USC_ID VARCHAR2(32) NULL ,
           ZB VARCHAR2(5) DEFAULT ''%'' NOT NULL ,
           EXTERNE_MELDUNG CLOB NULL ,
           BESTAETIGUNG NUMBER(10) DEFAULT 0 NOT NULL ,
           A0101_501_SID NUMBER(10) DEFAULT -1 NOT NULL ,
           UPD_USER NUMBER(10) DEFAULT -1 NOT NULL,
           UPD_DATE DATE DEFAULT SYSDATE NULL,
           CONSTRAINT PK_D_USC$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_D_USC$BTS_ID ON D_USECASE (BTS_ID)');
    CrtIdx('CREATE INDEX IDX_D_USC$BTS_ID2 ON D_USECASE (BTS_ID2)');
    CrtIdx('CREATE INDEX IDX_D_USC$USC_ID ON D_USECASE (USC_ID)');
    CrtIdx('CREATE INDEX IDX_D_USC$D0101_ID ON D_USECASE (D0101_ID)');
    CrtIdx('CREATE INDEX IDX_D_USC$A0101_501_SID ON D_USECASE (A0101_501_SID)');
    Comment_Tab('D_USECASE', 'Usecase (Krzel USC)');
    Comment_Col('D_USECASE.BTS_ID', 'neue BTS_ID');
    Comment_Col('D_USECASE.BTS_ID2', 'alte BTS_ID zum Vergleich oder Dubletten beim Usecase DUBLETTE');
    Comment_Col('D_USECASE.USECASE', 'Verweis auf VIS_A0101 SKZ 502 Feld CODE');
    Comment_Col('D_USECASE.USER_NAME', 'Ersteller des Usecase, S50.bez wenn USER_SID zugeordnet sonst den externen Namen');
    Comment_Col('D_USECASE.USER_SID', 'Ersteller des Usecase (S50.SID)');
    Comment_Col('D_USECASE.TYP', 'nderungsart: 0 - nderung, 1 - Korrektur, 2 - Lschen, 4 - Neu');
    Comment_Col('D_USECASE.AENDERUNGSGRUND', 'nderungsgrund');
    Comment_Col('D_USECASE.BEMERKUNG', 'Bemerkung zur nderung');
    Comment_Col('D_USECASE.DELETED', '-1 : besttigter Usecase');
    Comment_Col('D_USECASE.USC_ID', 'ID aus Ausgangskorb wenn aus iP gesendet (AUS_D_USECASE)');
    Comment_Col('D_USECASE.EXTERNE_MELDUNG', 'Meldungen vom WEB-Service oder Datenlieferanten');
    Comment_Col('D_USECASE.BESTAETIGUNG', 'Anzahl der Besttigungen, wenn grer 0 und DELETED = 0 dann ist die Besttigung durchgefhrt aber noch nicht abgeschlossen');
    Comment_Col('D_USECASE.A0101_501_SID', 'Katalog nderungsgrund');

  END IF;
  
  IF NOT TableExists('D_EXPORTSPERRE') THEN
    EXECUTE IMMEDIATE '         
      CREATE  TABLE D_EXPORTSPERRE
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           D0101_ID VARCHAR2(32) NOT NULL ,
           GESPERRT_JN NUMBER(1) DEFAULT 0 NOT NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL ,
           MANDANT NUMBER(10) NOT NULL ,
           USER_SID NUMBER(10) NOT NULL ,
           GESPERRT_BIS DATE NOT NULL      ,
           CONSTRAINT PK_D_EXS$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_D_EXS$01 ON D_EXPORTSPERRE (D0101_ID,GESPERRT_BIS,INS_DATE)');
    Comment_Tab('D_EXPORTSPERRE', 'gesperrte Betriebe (Krzel EXS)');
    Comment_Col('D_EXPORTSPERRE.ID', 'Betrieb gesperrt J/N');
    Comment_Col('D_EXPORTSPERRE.GESPERRT_JN', 'gesperrt J/N');
    Comment_Col('D_EXPORTSPERRE.GESPERRT_BIS', 'gesperrt bis');

  END IF;
  
  IF NOT TableExists('D_ABTEILUNG') THEN
    EXECUTE IMMEDIATE '         
      CREATE  TABLE D_ABTEILUNG
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           BTS_ID VARCHAR2(32) NOT NULL ,
           ADR_ID VARCHAR2(32) NULL ,
           TYP VARCHAR2(128) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           NUMMER VARCHAR2(64) NULL ,
           BEZEICHNUNG VARCHAR2(128) NULL ,
           D0103_ID VARCHAR2(32) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL      ,
           CONSTRAINT PK_D_ABT$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_D_ABT$BTS_ID ON D_ABTEILUNG (BTS_ID)');
    CrtIdx('CREATE INDEX IDX_D_ABT$ADR_ID ON D_ABTEILUNG (ADR_ID)');
    Comment_Tab('D_ABTEILUNG', 'Abteilungen (Krzel ABT)');
    Comment_Col('D_ABTEILUNG.ADR_ID', 'abweichende Adresse');
    Comment_Col('D_ABTEILUNG.TYP', 'Wert aus A0101_47.BEZ (Abteilungsart)');
    Comment_Col('D_ABTEILUNG.NUMMER', 'D0103.ABT_NUMMER');
    Comment_Col('D_ABTEILUNG.BEZEICHNUNG', 'Bezeichnung');
    Comment_Col('D_ABTEILUNG.D0103_ID', 'Referenz zu VIS_D0103.ID');
  END IF;
  
  IF NOT TableExists('D_FACHBEREICH') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE D_FACHBEREICH
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           BTS_ID VARCHAR2(32) NOT NULL ,
           CODE VARCHAR2(50) NOT NULL ,
           A0101_06_SID NUMBER(10) NULL ,
           GUELTIG_VON DATE NULL ,
           GUELTIG_BIS DATE NULL ,
           D010104_ID VARCHAR2(32) NULL ,
           A0115_SID NUMBER(10) NULL ,
           A0101_01_SID NUMBER(10) NULL ,
           ERSTKONTROLLTERMIN NUMBER(10) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NOT NULL      ,
           CONSTRAINT PK_D_FAB$ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_D_FAB$BTS_ID ON D_FACHBEREICH (BTS_ID)');
    Comment_Tab('D_FACHBEREICH', 'Fachbereiche (Krzel FAB)');
    Comment_Col('D_FACHBEREICH.CODE', 'A0101_47.BEZ');
    Comment_Col('D_FACHBEREICH.A0101_06_SID', 'Refeferenz auf Katalog Fachbereiche');
    Comment_Col('D_FACHBEREICH.GUELTIG_VON', 'gltig von');
    Comment_Col('D_FACHBEREICH.GUELTIG_BIS', 'gltig bis');
    Comment_Col('D_FACHBEREICH.D010104_ID', 'Referenz zu VIS_D010104.ID');
    Comment_Col('D_FACHBEREICH.A0115_SID', 'Referenz auf Katalog Risikobeurteilungsart');
    Comment_Col('D_FACHBEREICH.A0101_01_SID', 'Referenz auf Katalog Kontrollfristart');
    Comment_Col('D_FACHBEREICH.ERSTKONTROLLTERMIN', 'Erstkontrolltermin in Tagen');

  END IF;

  IF NOT TableExists('WKD_BEA') THEN
    EXECUTE IMMEDIATE ' 
      CREATE  TABLE WKD_BEA
      ( 
           ID VARCHAR2(32) DEFAULT SYS_GUID() NOT NULL ,
           MANDANT NUMBER(10) NULL ,
           CODE VARCHAR2(128) NULL ,
           CODEN NUMBER(10) NULL ,
           BEZ VARCHAR2(128) NULL ,
           BEM VARCHAR2(2000) NULL ,
           INS_USER NUMBER(10) NULL ,
           INS_DATE DATE DEFAULT SYSDATE NULL ,
           UPD_USER NUMBER(10) NULL ,
           UPD_DATE DATE DEFAULT SYSDATE NULL ,
           DELETED NUMBER(10) DEFAULT 0 NOT NULL ,
           DEL_USER NUMBER(10) NULL ,
           DEL_DATE DATE NULL ,
           D0101_ID VARCHAR2(32) NOT NULL ,
           RESULT NUMBER(10) DEFAULT 0 NOT NULL ,
           RESULT_MSG VARCHAR2(4000) NULL ,
           A0101_03_SID0 NUMBER(10) NULL ,
           A0101_03_SID1 NUMBER(10) NULL ,
           A0101_03_SID2 NUMBER(10) NULL ,
           A0101_03_CHK NUMBER(1) NULL ,
           BEZ_ART0 VARCHAR(1) NULL ,
           BEZ_ART1 VARCHAR(1) NULL,
           BEZ0 VARCHAR(128) NULL ,
           BEZ1 VARCHAR(128) NULL,
           BEZ1_FIELD VARCHAR2(30) NULL,
           BEZ_CHK NUMBER(1),
           P_NAME_0 VARCHAR2(128) NULL ,
           P_NAME_1 VARCHAR2(128) NULL ,     
           P_NAME1_0 VARCHAR2(256) NULL ,
           P_NAME1_1 VARCHAR2(256) NULL ,           
           P_NAME_CHK NUMBER(1) NULL ,
           VNAME_0 VARCHAR2(250) NULL ,
           VNAME_1 VARCHAR2(250) NULL ,
           NNAME_0 VARCHAR2(250) NULL ,
           NNAME_1 VARCHAR2(250) NULL ,
           NAME1_0 VARCHAR2(256) NULL ,
           NAME1_1 VARCHAR2(256) NULL ,
           NAME1_CHK NUMBER(1) NULL ,  
           ANREDE_0  VARCHAR2(50) NULL ,
           ANREDE_1 VARCHAR2(50) NULL ,
           TITEL_0 VARCHAR2(50) NULL ,
           TITEL_1 VARCHAR2(50) NULL,  
           
           CONSTRAINT PK_WKD_BEA_ID PRIMARY KEY (ID) NOT DEFERRABLE Initially IMMEDIATE ' || vUI_TBS || '
      )';

    CrtIdx('CREATE INDEX IDX_WKD_BEA_MANDANT$DAT ON WKD_BEA (MANDANT,UPD_DATE,DEL_DATE)');
    CrtIdx('CREATE UNIQUE INDEX IDX_WKD_BEA_D0101_ID ON WKD_BEA (D0101_ID)');
    Comment_Tab('WKD_BEA', 'Prfungen fr BEA');
    Comment_Col('WKD_BEA.DELETED', '0 - Standard, 1 - inaktiv, 2 - gelscht');
    Comment_Col('WKD_BEA.RESULT', 'Ergebnis der Prfung: 0 Betrieb muss berichtigt werden, 1 Betrieb fr Bearbeitung in BEA in Ordnung,');
    Comment_Col('WKD_BEA.RESULT_MSG', 'Ergebnisprfung als Text');
    Comment_Col('WKD_BEA.A0101_03_SID0', 'Originale Rechtsform');
    Comment_Col('WKD_BEA.A0101_03_SID1', 'Rechtsform aus Prfung');
    Comment_Col('WKD_BEA.A0101_03_SID2', 'Manuelle Korrektur der Rechtsform');
    Comment_Col('WKD_BEA.A0101_03_CHK', '0 originale Rechtsform bleibt erhalten, 1 Vorschlag Rechtsform wird bernommen, 2 manuelle Korrektur der Rechstform');
    Comment_Col('WKD_BEA.BEZ_ART0', 'original BEZ_ART');
    Comment_Col('WKD_BEA.BEZ_ART1', 'Vorschlag BEZ_ART');
    Comment_Col('WKD_BEA.BEZ0', 'original BEZ');
    Comment_Col('WKD_BEA.BEZ1', 'Vorschlag BEZ');
    Comment_Col('WKD_BEA.BEZ1_FIELD', 'Feld was zu BEZ1 fhrte');
    Comment_Col('WKD_BEA.BEZ_CHK', '0 originale BEZ bleibt erhalten, 1 BEZ1 wird als BEZ bernommen');
    Comment_Col('WKD_BEA.P_NAME_0', 'original P_NAME');
    Comment_Col('WKD_BEA.P_NAME_1', 'Vorschlag P_NAME');
    Comment_Col('WKD_BEA.P_NAME1_0', 'original P_NAME1');
    Comment_Col('WKD_BEA.P_NAME1_1', 'Vorschlag P_NAME1');
    Comment_Col('WKD_BEA.P_NAME_CHK', '1: Vorschlag P_NAME bernehmen, 2: P_NAME zustzl in P_NAME1 bernehmen');
    Comment_Col('WKD_BEA.VNAME_0', 'original VNAME');
    Comment_Col('WKD_BEA.VNAME_1', 'Vorschlag VNAME');
    Comment_Col('WKD_BEA.NNAME_0', 'original NNAME');
    Comment_Col('WKD_BEA.NNAME_1', 'Vorschlag NNAME');
    Comment_Col('WKD_BEA.NAME1_CHK', '1: Vorschlag NAME1 bernehmen');
    Comment_Col('WKD_BEA.TITEL_0', 'original TITEL');
    Comment_Col('WKD_BEA.TITEL_1', 'Vorschlag TITEL');
    Comment_Col('WKD_BEA.ANREDE_0', 'original ANREDE');
    Comment_Col('WKD_BEA.ANREDE_1', 'Vorschlag ANREDE');
  END IF;


  IF Not ConstraintExists('FK_D_BAR$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSART ADD
      (
           CONSTRAINT FK_D_BAR$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_BAR$ADR_STO_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSART ADD
      (
           CONSTRAINT FK_D_BAR$ADR_STO_ID FOREIGN KEY
           ( ADR_STO_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_BTS$ADR_STO_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSSTAETTE ADD
      (
           CONSTRAINT FK_D_BTS$ADR_STO_ID FOREIGN KEY
           ( ADR_STO_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_BTS$ADR_PST_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSSTAETTE ADD
      (
           CONSTRAINT FK_D_BTS$ADR_PST_ID FOREIGN KEY
           ( ADR_PST_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_BTS$ADR_BTR_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSSTAETTE ADD
      (
           CONSTRAINT FK_D_BTS$ADR_BTR_ID FOREIGN KEY
           ( ADR_BTR_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_BTS$ADR_STD_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSSTAETTE ADD
      (
           CONSTRAINT FK_D_BTS$ADR_STD_ID FOREIGN KEY
           ( ADR_STD_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PFK$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONALFUNKTION ADD
      (
           CONSTRAINT FK_D_PFK$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PFK$PRS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONALFUNKTION ADD
      (
           CONSTRAINT FK_D_PFK$PRS_ID FOREIGN KEY
           ( PRS_ID )
           REFERENCES D_PERSONAL
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PRS$OWNER_PRS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONAL ADD
      (
           CONSTRAINT FK_D_PRS$OWNER_PRS_ID FOREIGN KEY
           ( OWNER_PRS_ID )
           REFERENCES D_PERSONAL
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PRS$ADR_STZ_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONAL ADD
      (
           CONSTRAINT FK_D_PRS$ADR_STZ_ID FOREIGN KEY
           ( ADR_STZ_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PRS$ADR_PST_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONAL ADD
      (
           CONSTRAINT FK_D_PRS$ADR_PST_ID FOREIGN KEY
           ( ADR_PST_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_PRT$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PROTOKOLL ADD
      (
           CONSTRAINT FK_D_PRT$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_REG$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_REGISTRIERUNG ADD
      (
           CONSTRAINT FK_D_REG$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_REG$PRS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_REGISTRIERUNG ADD
      (
           CONSTRAINT FK_D_REG$PRS_ID FOREIGN KEY
           ( PRS_ID )
           REFERENCES D_PERSONAL
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_USC$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_USECASE ADD
      (
           CONSTRAINT FK_D_USC$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_USC$BTS_ID2') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_USECASE ADD
      (
           CONSTRAINT FK_D_USC$BTS_ID2 FOREIGN KEY
           ( BTS_ID2 )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_USC$USC_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_USECASE ADD
      (
           CONSTRAINT FK_D_USC$USC_ID FOREIGN KEY
           ( USC_ID )
           REFERENCES D_USECASE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_ABT$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_ABTEILUNG ADD
      (
           CONSTRAINT FK_D_ABT$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_ABT$ADR_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_ABTEILUNG ADD
      (
           CONSTRAINT FK_D_ABT$ADR_ID FOREIGN KEY
           ( ADR_ID )
           REFERENCES D_ADRESSE
           ( ID )
           NOT DEFERRABLE Initially IMMEDIATE
      )';
    END IF;     

  IF Not ConstraintExists('FK_D_FAB$BTS_ID') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_FACHBEREICH ADD
      (
           CONSTRAINT FK_D_FAB$BTS_ID FOREIGN KEY
           ( BTS_ID )
           REFERENCES D_BETRIEBSSTAETTE
           ( ID )
           ON DELETE CASCADE
           NOT DEFERRABLE Initially IMMEDIATE
      )';
  END IF;   
    
  -- 14.05.2014 Feldlngenanpassungen entsprechend vis_d0101
  SELECT COUNT(*) INTO vCnt
  FROM user_tab_columns d1
  WHERE table_name = 'D_PERSONAL'
  AND column_name = 'BEZEICHNUNG2'
  AND data_length < 255;
  
  IF vCnt > 0 THEN
    EXECUTE IMMEDIATE 'ALTER TABLE d_personal MODIFY bezeichnung2 VARCHAR2(255)';  
  END IF;
    
  SELECT COUNT(*) INTO vCnt
  FROM user_tab_columns d1
  WHERE table_name = 'D_ADRESSE'
  AND column_name = 'GEMARKUNG'
  AND data_length < 64;
  
  IF vCnt > 0 THEN
    EXECUTE IMMEDIATE 'ALTER TABLE d_adresse MODIFY (gemarkung VARCHAR2(64), flurstueck VARCHAR2(50), flurnummer VARCHAR2(50) )';  
  END IF;  
  
  -- 30.05.2014
  IF NOT IdxExists('IDX_D_PRT$STATUS$01') THEN
    CrtIdx('CREATE INDEX IDX_D_PRT$STATUS$01 ON D_PROTOKOLL ( STATUS, DELETED, DATENHERKUNFT, UPD_DATE )' );
  END IF;
  
  -- 18.06.2014
  -- todo 54627090
  SELECT COUNT(*) INTO vCnt
  FROM user_tab_columns d1
  WHERE table_name = 'D_PERSONAL'
  AND column_name IN( 'KURZBEZEICHNUNG1', 'KURZBEZEICHNUNG2')
  AND data_length > 30;  
  
  IF vCnt > 0 THEN
    EXECUTE IMMEDIATE '
      UPDATE d_personal  
      SET kurzbezeichnung1 = SUBSTR(kurzbezeichnung1, 1,30)
        , kurzbezeichnung2 = SUBSTR(kurzbezeichnung2, 1,30)
      WHERE LENGTH(kurzbezeichnung1) > 30 
      OR LENGTH(kurzbezeichnung1) > 30';
       
    EXECUTE IMMEDIATE ' 
      ALTER TABLE d_personal MODIFY (
        kurzbezeichnung1 VARCHAR2(30),
        kurzbezeichnung2 VARCHAR2(30) )';     
  END IF;

  --26.06.2014
  IF Not FieldExists('D_BETRIEBSSTAETTE', 'BEMERKUNG') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_BETRIEBSSTAETTE ADD (BEMERKUNG VARCHAR2(4000))';
  END IF;
  
  IF Not FieldExists('D_PERSONAL', 'BEMERKUNG') THEN
    EXECUTE IMMEDIATE ' 
      ALTER TABLE D_PERSONAL ADD (BEMERKUNG VARCHAR2(4000))';
  END IF;  
  
  IF NOT ConstraintExists('CST_D_PRS$OWNER_TYPE') THEN
    EXECUTE IMMEDIATE q'[
      ALTER TABLE D_PERSONAL ADD (
           CONSTRAINT CST_D_PRS$OWNER_TYPE CHECK (OWNER_TYPE IS NULL OR OWNER_TYPE IN ('NP', 'BTR' ))  NOT DEFERRABLE Initially IMMEDIATE
      )]';     

    Comment_Col('D_PERSONAL.OWNER_TYPE', 'Funktionstyp der Beziehung:  Betreiber BTR / Personal NP');
  END IF;
  
  
  IF Not ConstraintExists('CST_D_PRS$TYP') THEN
    EXECUTE IMMEDIATE q'[ 
      ALTER TABLE D_PERSONAL ADD (
           CONSTRAINT CST_D_PRS$TYP CHECK (TYP IN ('LP', 'NP', 'JP')) NOT DEFERRABLE Initially IMMEDIATE
      )]';
  END IF;
  
  IF Not ConstraintExists('CST_D_ADD$TYP') THEN
    EXECUTE IMMEDIATE q'[ 
      ALTER TABLE D_ADRESSE ADD (
           CONSTRAINT CST_D_ADD$TYP CHECK (TYP IN ('S', 'P', 'F', 'V')) NOT DEFERRABLE Initially IMMEDIATE
      )]';
  END IF;
  
  IF Not ConstraintExists('CST_D_FKT$TYP') THEN
    EXECUTE IMMEDIATE q'[ 
      ALTER TABLE D_PERSONALFUNKTION ADD (
           CONSTRAINT CST_D_FKT$TYP CHECK (TYP IN ( 'BTR', 'HV', 'NP' )) NOT DEFERRABLE Initially IMMEDIATE
      )]';
  END IF;

  IF Not ConstraintExists('CST_D_PRT$DATENHERKUNFT') THEN
    EXECUTE IMMEDIATE q'[ 
      ALTER TABLE D_PROTOKOLL ADD (
           CONSTRAINT CST_D_PRT$DATENHERKUNFT CHECK (DATENHERKUNFT IN ('BEA', 'WEIN', 'AUGE' )) NOT DEFERRABLE Initially IMMEDIATE
      )]';
  END IF;
 
  IF Not ConstraintExists('CST_D_PRT$STATUS') THEN
    EXECUTE IMMEDIATE q'[ 
      ALTER TABLE D_PROTOKOLL ADD (
           CONSTRAINT CST_D_PRT$STATUS CHECK (STATUS IN ( 0, 1, 2, 3, 4, 5, 6, 10, 20 )) NOT DEFERRABLE Initially IMMEDIATE
      )]';

    Comment_Col('D_PROTOKOLL.STATUS', 
'Betriebsregister Ausgangskorb
0 - nicht verarbeitet, vor dem Vergleich, 
2 - gesendet, zum Vergleich, 
3 - nur zum Vergleich, wird nicht und wurde nicht gesendet, 
4 - nur zum Vergleich mit dem Eingangskorb, 
5 - neuer Betrieb ohne Dublettenabgleich senden, 
6 - ohne Plausiprfung

Betriebsregister Eingangskorb
0 - externe nderungen/Neuanlagen,  
1 - Besttigung eigenen nderungen/Neuanlagen, 
2 - Besttigung ANL_MIN als Dublette, 
10 - Ablehnung, 
20 - Betrieb als Dublette

BEA:
0 - neuer Betrieb/Betreiber
1 - nderung Betreibe/Betreiber');

  END IF; 
  
  IF FieldExists('D_REGISTRIERUNG', 'ZUISATZ') THEN
    EXECUTE IMMEDIATE 'ALTER TABLE D_REGISTRIERUNG RENAME COLUMN ZUISATZ TO ZUSATZ';
  END IF;

  -- 30.09.2014
  Comment_Col('D_BETRIEBSART.GELTUNGSBEREICH', 'HBA - Kennzeichnung Hauptbetriebsart');

  -- todo 62080090
  SELECT COUNT(*) INTO vCnt
  FROM user_tab_columns d1
  WHERE table_name = 'D_PERSONAL'
  AND column_name IN( 'VORSATZ_NACHNAME', 'NACHSATZ_NACHNAME')
  AND data_length < 30; 

  FOR c1 IN (
    SELECT tab.table_name, tab.column_name
    FROM user_tab_columns tab
    WHERE tab.char_length < 30
    AND table_name = 'D_PERSONAL'
    AND tab.column_name IN ( 'VORSATZ_NACHNAME', 'NACHSATZ_NACHNAME')
  ) LOOP
    EXECUTE IMMEDIATE 'ALTER TABLE ' || c1.table_name || ' MODIFY ( ' || c1.column_name || ' VARCHAR2(30 CHAR) )';  
  END LOOP;
  
  -- 14.03.2017
  FOR c1 IN (
    SELECT d1.table_name, d1.column_name, d1.data_length
    FROM user_tab_columns d1
    WHERE d1.table_name IN ('D_PERSONAL')
    AND d1.column_name = 'GEBURTSORT'
    AND d1.char_length < 60
  ) LOOP
    EXECUTE IMMEDIATE 'ALTER TABLE '|| c1.table_name ||' MODIFY ' ||c1.column_name|| ' VARCHAR2(60 CHAR)';
  END LOOP;  
  
END;
/
